//----------------------------------------------------------------------------
/*
 * File: timer.c
 * - functions fo manage with services through timer
 * 
 * Part of Emulation of PMD 32 drive.
 * Based on MMC/SD/SDHC card library by Roland Riegel
 *   - http://www.roland-riegel.de/sd-reader/
 *
 * Author: Roman Brik { pmd85emu (at) gmail.com }
 * Web: http://pmd85.borik.net
 * Date: 10 Oct 2009
 * Version 2.0
*/
//----------------------------------------------------------------------------
#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>

#include "timer.h"
//----------------------------------------------------------------------------
#define TIMER_SERVICES_MAX    1

// prescaler config
// 1:1, 2:8, 3:32, 4:64, 5:128, 6:256, 7:1024 - ATmega128
// 1:1, 2:8, 3:64, 4:256, 5:1024, 6:ExtT0L, 7:ExtT0H - ATmega32
#define TIMER_PRESCALER_CFG   3
#define TIMER_PRESCALER       64
#define TIMER_OCR0            ((F_CPU / TIMER_PRESCALER / (1000 / TIMER_INTERVAL_MS)) - 1)
#if (TIMER_OCR0 == 0 || TIMER_OCR0 > 255)
  #error "Bad value of macro TIMER_OCR0"
#endif
//----------------------------------------------------------------------------
static volatile uint32_t total_ticks;
static uint8_t services_count;
static timer_service_handler_t service_handlers[TIMER_SERVICES_MAX];
//----------------------------------------------------------------------------
void timer_init(void)
{
  // set-up TIMER 0
  TIMSK &= ~(_BV(TOIE0) | _BV(OCIE0));
  ASSR = 0;
  TCCR0 = (_BV(WGM01) | TIMER_PRESCALER_CFG);
	OCR0 = TIMER_OCR0;
  TIMSK |= _BV(OCIE0);

  total_ticks = 0;
  services_count = 0;

  for (uint8_t ii = 0; ii < TIMER_SERVICES_MAX; ii++)
    service_handlers[ii] = 0;
}
//----------------------------------------------------------------------------
uint32_t timer_get_ticks(void)
{
  return total_ticks;
}
//----------------------------------------------------------------------------
void timer_wait_msecs(uint16_t msecs)
{
  uint32_t end_ticks;

  if (msecs == 0)
    return;

  end_ticks = total_ticks + msecs;

  do {
    sleep_mode();
  } while (total_ticks < end_ticks);
}
//----------------------------------------------------------------------------
void timer_wait_secs(uint8_t secs)
{
  uint32_t end_ticks;

  if (secs == 0)
    return;

  end_ticks = total_ticks + (secs * (1000 / TIMER_INTERVAL_MS));

  do {
    sleep_mode();
  } while (total_ticks < end_ticks);
}
//----------------------------------------------------------------------------
void timer_add_service(timer_service_handler_t handler)
{
  if (handler == 0)
    return;

  for (uint8_t ii = 0; ii < TIMER_SERVICES_MAX; ii++)
  {
    if (service_handlers[ii] == 0)
    {
      service_handlers[ii] = handler;
      break;
    }
  }
}
//----------------------------------------------------------------------------
void timer_remove_service(timer_service_handler_t handler)
{
  if (handler == 0)
    return;

  for (uint8_t ii = 0; ii < TIMER_SERVICES_MAX; ii++)
  {
    if (service_handlers[ii] == handler)
    {
      service_handlers[ii] = 0;
      break;
    }
  }
}
//----------------------------------------------------------------------------
//ISR(TIMER0_OVF_vect )
ISR(TIMER0_COMP_vect)
{
  total_ticks++;

  for (uint8_t ii = 0; ii < TIMER_SERVICES_MAX; ii++)
  {
    if (service_handlers[ii] != 0)
      service_handlers[ii](total_ticks);
  }
}
//----------------------------------------------------------------------------
