//----------------------------------------------------------------------------
/*
 * File: ui.c
 * - user interface functions
 * 
 * Part of Emulation of PMD 32 drive.
 * Based on MMC/SD/SDHC card library by Roland Riegel
 *   - http://www.roland-riegel.de/sd-reader/
 *
 * Author: Roman Brik { pmd85emu (at) gmail.com }
 * Web: http://pmd85.borik.net
 * Date: 10 Oct 2009
 * Version 2.0
*/
//----------------------------------------------------------------------------
#include <inttypes.h>
#include <string.h>
#include <avr/io.h>
#include <util/delay.h>

#include "pgm_strings.h"
#include "fat.h"
#include "lcd.h"
#include "btn.h"
#include "apputil.h"
#include "ui.h"

#if (0 & DEBUG)
  #include "uart.h"
  #define DEBUG_UI  1
#else
  #define DEBUG_UI  0
#endif
//----------------------------------------------------------------------------
static char str[LCD_DISPLAY_COLUMNS + 1];
//----------------------------------------------------------------------------
void ui_wait_please(PGM_P title)
{
  lcd_disp_string_at(apputil_center_text_p(str, title, LCD_DISPLAY_COLUMNS), 0, 0);
  lcd_disp_string_at(apputil_center_text_p(str, pstr_wait_please, LCD_DISPLAY_COLUMNS), 1, 0);
}
//----------------------------------------------------------------------------
uint8_t ui_show_message(PGM_P line1, PGM_P line2)
{
  lcd_disp_string_at(apputil_center_text_p(str, line1, LCD_DISPLAY_COLUMNS), 0, 0);
  lcd_disp_string_at(apputil_center_text_p(str, line2, LCD_DISPLAY_COLUMNS), 1, 0);
  return btn_read();
}
//----------------------------------------------------------------------------
int8_t ui_menu(PGM_VOID_P menu, int8_t item_num)
{
  uint8_t b;
  int8_t ic;
  PGM_P txt;
  uint16_t adr;

  if (menu == 0)
    return -1;

  ic = pgm_read_byte(menu + 2);
  if (ic < 1)
    return -1;

  adr = pgm_read_word(menu + 3);
  for (b = 0; b < ic; b++)
  {
    if (item_num == pgm_read_byte(adr + b * sizeof(MENU_ITEM) + 2))
    {
      item_num = b;
      break;
    }
  }
  if (b >= ic)
    item_num = 0;

  txt = (PGM_P) pgm_read_word(menu);
  lcd_disp_string_at(apputil_center_text_p(str, txt, LCD_DISPLAY_COLUMNS), 0, 0);

  while (1)
  {
    txt = (PGM_P) pgm_read_word(adr + item_num * sizeof(MENU_ITEM));
    lcd_disp_string_at(apputil_left_text_p(str, txt, LCD_DISPLAY_COLUMNS), 1, 0);

    b = btn_read();
    switch (b)
    {
      case BTN_SET :
        return pgm_read_byte(adr + item_num * sizeof(MENU_ITEM) + 2);

      case BTN_CANCEL :
        return -1;

      case BTN_LEFT :
        item_num--;
        if (item_num < 0)
          item_num = ic - 1;
        break;

      case BTN_RIGHT :
        item_num++;
        if (item_num >= ic)
          item_num = 0;
        break;
    }
  }
}
//----------------------------------------------------------------------------
#define FAT_ATTRIB_ROOT     (1 << 7)
uint8_t ui_select_file(PGM_P title, struct fat_fs_struct *fs, PGM_P ext,
                       char *out_file_path, uint8_t max_path_len, uint8_t dirs)
{
  char *p;
  uint8_t dir_reset, b, d, exit, l;
  struct fat_dir_entry_struct de;
  struct fat_dir_struct* dd, *dd_new;

  if (fs == 0 || out_file_path == 0 || max_path_len == 0)
    return 0;

  lcd_disp_string_at(apputil_center_text_p(str, title, LCD_DISPLAY_COLUMNS), 0, 0);
  
  fat_get_dir_entry_of_path(fs, "/", &de);
  l = 0;

  *out_file_path = 0;
  d = 0;

  dd = fat_open_dir(fs, &de);
  if (!dd)
    return 0; // fs error

  while (1)
  {
    fat_reset_dir(dd);
    dir_reset = 1;

    exit = 0;
    b = BTN_CANCEL;
    while (1)
    {
      if (dirs && dir_reset && *out_file_path == 0)
      {
        apputil_left_text_p(str, pstr_root, LCD_DISPLAY_COLUMNS);
        de.attributes = FAT_ATTRIB_ROOT;
        d = 1;
      }
      else {
        if (b > 0 && fat_read_dir(dd, &de) == 0)
        {
          if (dir_reset)
          {
            lcd_disp_string_at(apputil_center_text_p(str, pstr_no_files_folders,
                                                     LCD_DISPLAY_COLUMNS), 1, 0);
            btn_read();
            exit = 1;
          }

          break; // jump reset dir
        }

        if (de.attributes != FAT_ATTRIB_ROOT)
        {
          if (de.attributes & FAT_ATTRIB_VOLUME)
            continue;
          else if (de.attributes & FAT_ATTRIB_HIDDEN)
            continue;
          else if (de.attributes & FAT_ATTRIB_DIR)
          {
            if (strcmp(de.long_name, ".") == 0)
              continue;

            if (strcmp(de.long_name, "..") == 0)
            {
              memset(str, ' ', LCD_DISPLAY_COLUMNS);
              str[LCD_DISPLAY_COLUMNS] = 0;
              p = str;
              d = l;
              if (d > LCD_DISPLAY_COLUMNS - 2)
                d = LCD_DISPLAY_COLUMNS - 2;
              while (d-- > 1)
                *p++ = '/';
              *p++ = '.';
              *p++ = '.';
              d = 2;  // dir - level up
            }
            else
            {
              d = 1;  // dir nesting
              str[0] = '[';
              apputil_left_text(str + 1, de.long_name, LCD_DISPLAY_COLUMNS - 1);
              if (strlen(de.long_name) < (LCD_DISPLAY_COLUMNS - 2))
                str[strlen(de.long_name) + 1] = ']';
              else
                str[LCD_DISPLAY_COLUMNS - 1] = ']';
            }
          }
          else
          {
            if (dirs)
              continue;
            if (ext != 0 && !apputil_check_file_ext_p(de.long_name, ext))
              continue;
            d = 0;  // file
            apputil_left_text(str, de.long_name, LCD_DISPLAY_COLUMNS);
          }
        }
      }
      lcd_disp_string_at(str, 1 , 0);
      dir_reset = 0;

      b = btn_read();
      if (b == BTN_CANCEL) // cancel selection
      {
        exit = 1;
        break;
      }

      if (b == BTN_SET || b == BTN_LEFT) // select file/dir, dir nesting
      {
        if (d == 2) // dir - level up
        {
          if (b == BTN_SET)
          {
            b = 0;
            continue;
          }
          p = strrchr(out_file_path, '/');
          if (p == 0)
            *out_file_path = 0;
          else
            *p = 0;
          l--;
        }
        else if (de.attributes != FAT_ATTRIB_ROOT)
        {
          if ((strlen(out_file_path) + strlen(de.long_name) + 1) >= max_path_len)
          {
            lcd_disp_string_at(apputil_center_text_p(str, pstr_path_too_long,
                                                     LCD_DISPLAY_COLUMNS), 1, 0);
            btn_read();
            b = 0;
            continue;
          }
          else
          {
            if (*out_file_path)
              strcat(out_file_path, "/");
            strcat(out_file_path, de.long_name);
            l++;
          }
        }

        if (b == BTN_SET && ((!dirs && !d) || (dirs && d)))
        {
          if (*out_file_path == 0)
            strcat(out_file_path, "/");
          exit = 2;
#if DEBUG_UI
          uart_puts_p(PSTR("Selected file/dir:"));
          uart_puts(out_file_path);
          uart_putc('\n');
#endif
          break; // ok, file/dir selected
        }

        if (de.attributes == FAT_ATTRIB_ROOT
            || (b == BTN_LEFT && d == 0))
          b = 0;
        else
        {
          dd_new = fat_open_dir(fs, &de);
          if (dd_new)
          {
            fat_close_dir(dd);
            dd = dd_new;
          }
          else
            exit = 1; // fs error
          break;
        }
      }

//      if (b == BTN_RIGHT) // next item
//        continue;
    }

    if (exit)
      break;
  }

  if (dd)
    fat_close_dir(dd);

  return exit - 1;
}
//----------------------------------------------------------------------------
uint8_t ui_input_string(PGM_P title, char *out, uint8_t max_len)
{
  char c, *p;
  uint8_t b, off, pos, exit;

  if (out == 0 || max_len == 0)
    return 0;

  lcd_disp_string_at(apputil_center_text_p(str, title, LCD_DISPLAY_COLUMNS), 0, 0);

  *out = ' ';
  *(out + 1) = 0;
  p = out;
  off = 0;
  pos = 0;
  exit = 0;

  lcd_cursor_on();
  while (1)
  {
    lcd_disp_string_at(apputil_left_text(str, p, LCD_DISPLAY_COLUMNS), 1, 0);
    while (1)
    {
      lcd_set_position(1, pos);

      b = btn_read();
      if (b == BTN_CANCEL) // cancel input
      {
        exit = 1;
        break;
      }
      else if (b == BTN_SET) // accept input
      {
        exit = 2;
        break;
      }
      else if (b == BTN_LEFT)
      {
        c = out[off] + 1;
        if (c == (' ' + 1) || c == ('9' + 1))
          c = '.';
        else if (c == ('.' + 1))
          c = '-';
        else if (c == ('-' + 1))
          c = '_';
        else if (c == ('_' + 1))
          c = 'a';
        else if (c == ('z' + 1))
          c = 'A';
        else if (c == ('Z' + 1))
          c = '0';
        out[off] = c;

        lcd_disp_char(c);
      }
      else if (b == BTN_RIGHT)
      {
        if (out[off] == ' ')
        {
          p = out;
          off = 0;
          pos = 0;
          break;
        }
        else if (off < max_len - 1)
        {
          off++;
          if (out[off] == 0)
          {
            out[off] = ' ';
            out[off + 1] = 0;
          }
        }

        if (off < LCD_DISPLAY_COLUMNS)
        {
          p = out;
          pos = off;
        }
        else
        {
          p = out + off - (LCD_DISPLAY_COLUMNS - 1);
          pos = (LCD_DISPLAY_COLUMNS - 1);
          break;
        }
      }
    }

    if (exit)
      break;
  }

  apputil_trim(out);
  lcd_cursor_off();

#if DEBUG_UI
  if (exit == 2)
  {
    uart_puts_p(PSTR("String:"));
    uart_puts(out);
    uart_putc('\n');
  }
#endif

  return exit - 1;
}
//----------------------------------------------------------------------------
